import os
from collections.abc import Generator
from contextlib import contextmanager
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Any
from unittest import mock

from flask import Flask, jsonify, request
from flask.typing import ResponseReturnValue

from kagglehub.clients import (
    ColabClient,
)
from kagglehub.colab_cache_resolver import COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME

app = Flask(__name__)

LATEST_MODEL_VERSION = 2


@app.route("/", methods=["HEAD"])
def head() -> ResponseReturnValue:
    return "", 200


@app.errorhandler(404)
def error(e: Exception):  # noqa: ANN201
    data = {"code": "404", "error": str(e)}
    return jsonify(data), 200


@app.route(ColabClient.IS_SUPPORTED_PATH, methods=["POST"])
def models_is_supported() -> ResponseReturnValue:
    data = request.get_json()
    version = LATEST_MODEL_VERSION
    if "version" in data:
        version = data["version"]
    model = data["model"]
    framework = data["framework"]
    variation = data["variation"]
    slug = f"{model}/{framework}/{variation}/{version}"
    if data["owner"] == "unavailable":
        return slug, 400
    return "", 200


@app.route(ColabClient.MOUNT_PATH, methods=["POST"])
def models_mount() -> ResponseReturnValue:
    data = request.get_json()
    version = LATEST_MODEL_VERSION
    if "version" in data:
        version = data["version"]
    model = data["model"]
    framework = data["framework"]
    variation = data["variation"]
    slug = f"{model}/{framework}/{variation}/{version}"
    cache_mount_folder = os.getenv(COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME)
    base_path = f"{cache_mount_folder}/{slug}"
    os.makedirs(base_path, exist_ok=True)
    Path(f"{base_path}/config.json").touch()
    if version == LATEST_MODEL_VERSION:
        # The latest version has an extra file.
        Path(f"{base_path}/model.keras").touch()
    return jsonify({"slug": slug}), 200


@contextmanager
def create_env() -> Generator[Any, Any, Any]:
    with TemporaryDirectory() as cache_mount_folder:
        with mock.patch.dict(
            os.environ,
            {
                COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME: cache_mount_folder,
            },
        ):
            yield



import functools
import inspect
import logging
import os
from importlib import metadata  # type: ignore
from typing import Optional

KAGGLE_NOTEBOOK_ENV_VAR_NAME = "KAGGLE_KERNEL_RUN_TYPE"
KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME = "KAGGLE_DATA_PROXY_URL"
KAGGLE_TOKEN_KEY_DIR_ENV_VAR_NAME = "KAGGLE_API_V1_TOKEN"

logger = logging.getLogger(__name__)


def is_in_colab_notebook() -> bool:
    return os.getenv("COLAB_RELEASE_TAG") is not None


def is_in_kaggle_notebook() -> bool:
    if os.getenv(KAGGLE_NOTEBOOK_ENV_VAR_NAME) is not None:
        if os.getenv(KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME) is None:
            # Missing endpoint for the Jwt client
            logging.warning(
                "Can't use the Kaggle Cache. "
                f"The '{KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME}' environment variable is not set."
            )
            return False
        return True
    return False


@functools.cache
def read_kaggle_build_date() -> str:
    build_date_file = "/etc/build_date"
    try:
        with open(build_date_file) as file:
            return file.read().strip()
    except FileNotFoundError:
        logging.warning(f"Build date file {build_date_file} not found in Kaggle Notebook environment.")
        return "unknown"


def search_lib_in_call_stack(lib_name: str) -> Optional[str]:
    """Search the call stack for a given library name and get its information.

    Args:
        lib_name (str):
            The name of the library to search for.
            We use str.startswith so the lib_name must match the exact module name from beginning.

    Returns:
        str: A formatted string f"{lib_name}/{lib_version}" if found, otherwise None.
    """
    for frame_info in inspect.stack():
        module = inspect.getmodule(frame_info.frame)
        if module and hasattr(module, "__name__"):
            module_name = module.__name__
        else:
            module_name = None

        if module_name is not None and module_name.startswith(lib_name):
            try:
                lib_version = metadata.version(lib_name)
                return f"{lib_name}/{lib_version}"
            except metadata.PackageNotFoundError:
                continue
    return None



import threading
from dataclasses import dataclass, field

from flask import Flask, jsonify, request
from flask.typing import ResponseReturnValue

from ..utils import resolve_endpoint

app = Flask(__name__)


@dataclass
class SharedData:
    files: list[str] = field(default_factory=list)
    simulate_308: bool = False
    blob_request_count: int = 0


shared_data: SharedData = SharedData()
lock = threading.Lock()


def _increment_blob_request() -> None:
    lock.acquire()
    shared_data.blob_request_count += 1
    lock.release()


def _add_file(file: str) -> None:
    lock.acquire()
    shared_data.files.append(file)
    lock.release()


def reset() -> None:
    lock.acquire()
    shared_data.files = []
    shared_data.blob_request_count = 0
    shared_data.simulate_308 = False
    lock.release()


def simulate_308(*, state: bool) -> None:
    lock.acquire()
    shared_data.simulate_308 = state
    lock.release()


@app.route("/", methods=["HEAD"])
def head() -> ResponseReturnValue:
    return "", 200


@app.route("/api/v1/datsets/view/<owner_slug>/<dataset_slug>", methods=["GET"])
def dataset_get(owner_slug: str, dataset_slug: str) -> ResponseReturnValue:
    data = {"message": f"Dataset exists {owner_slug}/{dataset_slug} !"}
    return jsonify(data), 200


@app.errorhandler(404)
def error(e: Exception):  # noqa: ANN201
    data = {"message": "Some response data", "error": str(e)}
    return jsonify(data), 404


@app.route("/api/v1/datasets/create/new", methods=["POST"])
def dataset_create() -> ResponseReturnValue:
    data = {"status": "success", "message": "Dataset created successfully"}
    return jsonify(data), 200


@app.route("/api/v1/datasets/create/version/<owner_slug>/<dataset_slug>", methods=["POST"])
def dataset_create_version(owner_slug: str, dataset_slug: str) -> ResponseReturnValue:
    data = {
        "status": "success",
        "message": f"Dataset Version {owner_slug}/{dataset_slug} created successfully",
    }
    return jsonify(data), 200


@app.route("/api/v1/blobs/upload", methods=["POST"])
def blob_upload() -> ResponseReturnValue:
    if shared_data.simulate_308 and shared_data.blob_request_count < 0:
        _increment_blob_request()
        return "", 308
    address, port = resolve_endpoint()
    data = {
        "token": "dummy",
        "createUrl": f"http://{address}:{port}/upload/storage/v1/b/kaggle-datasets-data/o",
        "status": "success",
        "message": "Here is your token and Url",
    }
    post_data = request.get_json()
    _add_file(post_data["name"])
    _increment_blob_request()
    return jsonify(data), 200


@app.route("/upload/storage/v1/b/kaggle-datasets-data/o", methods=["PUT"])
def dataset_up_create() -> ResponseReturnValue:
    data = {"status": "success", "message": "File uploaded"}
    return jsonify(data), 200



import os
from unittest import mock

import requests

import kagglehub
from kagglehub.config import DISABLE_KAGGLE_CACHE_ENV_VAR_NAME
from kagglehub.env import KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME
from tests.fixtures import BaseTestCase

from .server_stubs import jwt_stub as stub
from .server_stubs import serv

INVALID_ARCHIVE_DATASET_HANDLE = "invalid/invalid/invalid/invalid/invalid"
VERSIONED_DATASET_HANDLE = "sarahjeffreson/featured-spotify-artiststracks-with-metadata/versions/1"
UNVERSIONED_DATASET_HANDLE = "sarahjeffreson/featured-spotify-artiststracks-with-metadata"
LATEST_DATASET_VERSION = 2
TEST_FILEPATH = "foo.txt"


# Test cases for the DatasetKaggleCacheResolver.
class TestKaggleCacheDatasetDownload(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app, KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME, "http://localhost:7778")

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def test_unversioned_dataset_download(self) -> None:
        with stub.create_env():
            dataset_path = kagglehub.dataset_download(UNVERSIONED_DATASET_HANDLE)
            self.assertEqual(["bar.csv", "foo.txt"], sorted(os.listdir(dataset_path)))

    def test_versioned_dataset_download(self) -> None:
        with stub.create_env():
            dataset_path = kagglehub.dataset_download(VERSIONED_DATASET_HANDLE)
            self.assertEqual(["foo.txt"], sorted(os.listdir(dataset_path)))

    def test_versioned_dataset_download_with_path(self) -> None:
        with stub.create_env():
            dataset_file_path = kagglehub.dataset_download(VERSIONED_DATASET_HANDLE, "foo.txt")
            self.assertTrue(dataset_file_path.endswith("foo.txt"))
            self.assertTrue(os.path.isfile(dataset_file_path))

    def test_unversioned_dataset_download_with_path(self) -> None:
        with stub.create_env():
            dataset_file_path = kagglehub.dataset_download(UNVERSIONED_DATASET_HANDLE, "bar.csv")
            self.assertTrue(dataset_file_path.endswith("bar.csv"))
            self.assertTrue(os.path.isfile(dataset_file_path))

    def test_versioned_dataset_download_with_missing_file_raises(self) -> None:
        with stub.create_env():
            with self.assertRaises(ValueError):
                kagglehub.dataset_download(VERSIONED_DATASET_HANDLE, "missing.txt")

    def test_unversioned_dataset_download_with_missing_file_raises(self) -> None:
        with stub.create_env():
            with self.assertRaises(ValueError):
                kagglehub.dataset_download(UNVERSIONED_DATASET_HANDLE, "missing.txt")

    def test_kaggle_resolver_skipped(self) -> None:
        with mock.patch.dict(os.environ, {DISABLE_KAGGLE_CACHE_ENV_VAR_NAME: "true"}):
            with stub.create_env():
                # Assert that a ConnectionError is set (uses HTTP server which is not set)
                with self.assertRaises(requests.exceptions.ConnectionError):
                    kagglehub.dataset_download(VERSIONED_DATASET_HANDLE)

    def test_versioned_dataset_download_bad_handle_raises(self) -> None:
        with self.assertRaises(ValueError):
            kagglehub.dataset_download("bad handle")

    def test_versioned_dataset_download_with_force_download(self) -> None:
        with stub.create_env():
            dataset_path = kagglehub.dataset_download(VERSIONED_DATASET_HANDLE)
            dataset_path_forced = kagglehub.dataset_download(VERSIONED_DATASET_HANDLE, force_download=True)

            # Using force_download shouldn't change the expected output of model_download.
            self.assertEqual(["foo.txt"], sorted(os.listdir(dataset_path_forced)))
            self.assertEqual(dataset_path, dataset_path_forced)

    def test_versioned_dataset_download_with_force_download_explicitly_false(self) -> None:
        with stub.create_env():
            dataset_path = kagglehub.dataset_download(VERSIONED_DATASET_HANDLE, force_download=False)
            self.assertEqual(["foo.txt"], sorted(os.listdir(dataset_path)))



import os
from tempfile import TemporaryDirectory
from unittest.mock import MagicMock, patch

import kagglehub
from kagglehub import clients
from kagglehub.clients import KaggleApiV1Client
from kagglehub.exceptions import DataCorruptionError
from tests.fixtures import BaseTestCase

from .server_stubs import kaggle_api_stub as stub
from .server_stubs import serv


class TestKaggleApiV1Client(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app)

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def test_download_with_integrity_check(self) -> None:
        with TemporaryDirectory() as d:
            out_file = os.path.join(d, "out")

            api_client = KaggleApiV1Client()
            api_client.download_file("good", out_file)

            with open(out_file) as f:
                self.assertEqual("foo", f.read())

    def test_resumable_download_with_integrity_check(self) -> None:
        with TemporaryDirectory() as d:
            out_file = os.path.join(d, "out")

            # If the out_file already has data, we use the 'Range' header to resume download.
            with open(out_file, "w") as f:
                f.write("fo")  # Should download the remaining "o".

            api_client = KaggleApiV1Client()
            with self.assertLogs("kagglehub", level="INFO") as cm:
                api_client.download_file("good", out_file)
                self.assertIn("INFO:kagglehub.clients:Resuming download from 2 bytes (1 bytes left)...", cm.output)

            with open(out_file) as f:
                self.assertEqual("foo", f.read())

    def test_download_no_integrity_check(self) -> None:
        with TemporaryDirectory() as d:
            out_file = os.path.join(d, "out")

            api_client = KaggleApiV1Client()
            api_client.download_file("no-integrity", out_file)

            with open(out_file) as f:
                self.assertEqual("foo", f.read())

    def test_download_corrupted_file_fail_integrity_check(self) -> None:
        with TemporaryDirectory() as d:
            out_file = os.path.join(d, "out")

            api_client = KaggleApiV1Client()
            with self.assertRaises(DataCorruptionError):
                api_client.download_file("corrupted", out_file)

            # Assert the corrupted file has been deleted.
            self.assertFalse(os.path.exists(out_file))

    @patch.dict("os.environ", {})
    def test_get_user_agent(self) -> None:
        self.assertEqual(clients.get_user_agent(), f"kagglehub/{kagglehub.__version__}")

    @patch.dict(
        "os.environ", {"KAGGLE_KERNEL_RUN_TYPE": "Interactive", "KAGGLE_DATA_PROXY_URL": "https://dp.kaggle.net"}
    )
    def test_get_user_agent_kkb(self) -> None:
        self.assertEqual(clients.get_user_agent(), f"kagglehub/{kagglehub.__version__} kkb/unknown")

    @patch.dict(
        "os.environ",
        {
            "COLAB_RELEASE_TAG": "release-colab-20230531-060125-RC00",
        },
    )
    def test_get_user_agent_colab(self) -> None:
        self.assertEqual(
            clients.get_user_agent(),
            f"kagglehub/{kagglehub.__version__} colab/release-colab-20230531-060125-RC00-unmanaged",
        )

    @patch("importlib.metadata.version")
    @patch("inspect.ismodule")
    @patch("inspect.stack")
    def test_get_user_agent_keras_nlp(
        self, mock_stack: MagicMock, mock_is_module: MagicMock, mock_version: MagicMock
    ) -> None:
        # Mock the call stack and version information.
        mock_stack.return_value = [
            MagicMock(frame=MagicMock(__name__="kagglehub.clients")),
            MagicMock(frame=MagicMock(__name__="kagglehub.models_helpers")),
            MagicMock(frame=MagicMock(__name__="kagglehub.models")),
            MagicMock(frame=MagicMock(__name__="keras_nlp.src.utils.preset_utils")),
            MagicMock(frame=MagicMock(None)),
        ]
        mock_is_module.return_value = True
        mock_version.return_value = "0.15.0"
        self.assertEqual(clients.get_user_agent(), f"kagglehub/{kagglehub.__version__} keras_nlp/0.15.0")



import os
from pathlib import Path
from tempfile import TemporaryDirectory

from kagglehub.exceptions import BackendError
from kagglehub.gcs_upload import MAX_FILES_TO_UPLOAD, TEMP_ARCHIVE_FILE
from kagglehub.models import model_upload
from tests.fixtures import BaseTestCase

from .server_stubs import model_upload_stub as stub
from .server_stubs import serv

TEMP_TEST_FILE = "temp_test_file"
APACHE_LICENSE = "Apache 2.0"


class TestModelUpload(BaseTestCase):
    def setUp(self) -> None:
        stub.reset()

    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app)

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def test_model_upload_with_invalid_handle(self) -> None:
        with self.assertRaises(ValueError):
            with TemporaryDirectory() as temp_dir:
                test_filepath = Path(temp_dir) / TEMP_TEST_FILE
                test_filepath.touch()  # Create a temporary file in the temporary directory
                model_upload("invalid/invalid/invalid", temp_dir, APACHE_LICENSE)

    def test_model_upload_instance_with_valid_handle(self) -> None:
        with TemporaryDirectory() as temp_dir:
            test_filepath = Path(temp_dir) / TEMP_TEST_FILE
            test_filepath.touch()  # Create a temporary file in the temporary directory
            model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, APACHE_LICENSE)
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_TEST_FILE, stub.shared_data.files)

    def test_model_upload_instance_with_nested_directories(self) -> None:
        with TemporaryDirectory() as temp_dir:
            # Create a nested directory structure
            nested_dir = Path(temp_dir) / "nested"
            nested_dir.mkdir()
            # Create a temporary file in the nested directory
            test_filepath = nested_dir / TEMP_TEST_FILE
            test_filepath.touch()
            model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, APACHE_LICENSE)
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_TEST_FILE, stub.shared_data.files)

    def test_model_upload_version_with_valid_handle(self) -> None:
        with TemporaryDirectory() as temp_dir:
            test_filepath = Path(temp_dir) / TEMP_TEST_FILE
            test_filepath.touch()  # Create a temporary file in the temporary directory
            model_upload("metaresearch/llama-2/pyTorch/7b", temp_dir, APACHE_LICENSE)
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_TEST_FILE, stub.shared_data.files)

    def test_model_upload_with_too_many_files(self) -> None:
        with TemporaryDirectory() as temp_dir:
            # Create more than 50 temporary files in the directory
            for i in range(MAX_FILES_TO_UPLOAD + 1):
                test_filepath = Path(temp_dir) / f"temp_test_file_{i}"
                test_filepath.touch()
            model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, APACHE_LICENSE)
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_ARCHIVE_FILE, stub.shared_data.files)

    def test_model_upload_resumable(self) -> None:
        # Enable simulation of 308 response for this test
        stub.simulate_308(state=True)
        with TemporaryDirectory() as temp_dir:
            test_filepath = Path(temp_dir) / TEMP_TEST_FILE
            test_filepath.touch()
            with open(test_filepath, "wb") as f:
                f.write(os.urandom(1000))
            model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, APACHE_LICENSE)
            self.assertGreaterEqual(stub.shared_data.blob_request_count, 1)
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_TEST_FILE, stub.shared_data.files)

    def test_model_upload_with_none_license(self) -> None:
        with TemporaryDirectory() as temp_dir:
            test_filepath = Path(temp_dir) / TEMP_TEST_FILE
            test_filepath.touch()  # Create a temporary file in the temporary directory
            model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, None)
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_TEST_FILE, stub.shared_data.files)

    def test_model_upload_without_license(self) -> None:
        with TemporaryDirectory() as temp_dir:
            test_filepath = Path(temp_dir) / TEMP_TEST_FILE
            test_filepath.touch()  # Create a temporary file in the temporary directory
            model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, version_notes="some notes")
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_TEST_FILE, stub.shared_data.files)

    def test_model_upload_with_invalid_license_fails(self) -> None:
        with TemporaryDirectory() as temp_dir:
            test_filepath = Path(temp_dir) / TEMP_TEST_FILE
            test_filepath.touch()  # Create a temporary file in the temporary directory
            with self.assertRaises(BackendError):
                model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, "Invalid License")

    def test_single_file_upload(self) -> None:
        with TemporaryDirectory() as temp_dir:
            test_filepath = Path(temp_dir) / "single_dummy_file.txt"
            with open(test_filepath, "wb") as f:
                f.write(os.urandom(100))

            model_upload("metaresearch/new-model/pyTorch/new-variation", str(test_filepath), APACHE_LICENSE)

            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn("single_dummy_file.txt", stub.shared_data.files)

    def test_model_upload_with_directory_structure(self) -> None:
        with TemporaryDirectory() as temp_dir:
            base_path = Path(temp_dir)
            (base_path / "dir1").mkdir()
            (base_path / "dir2").mkdir()

            (base_path / "file1.txt").touch()

            (base_path / "dir1" / "file2.txt").touch()
            (base_path / "dir1" / "file3.txt").touch()

            (base_path / "dir1" / "subdir1").mkdir()
            (base_path / "dir1" / "subdir1" / "file4.txt").touch()

            model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, APACHE_LICENSE)

            self.assertEqual(len(stub.shared_data.files), 4)
            expected_files = {"file1.txt", "file2.txt", "file3.txt", "file4.txt"}
            self.assertTrue(set(stub.shared_data.files).issubset(expected_files))

            # TODO: Add assertions on CreateModelInstanceRequest.Directories and
            # CreateModelInstanceRequest.Files to verify the expected structure
            # is sent.

    def test_model_upload_with_ignore_patterns(self) -> None:
        with TemporaryDirectory() as tmp_dir:
            tmp_dir_p = Path(tmp_dir)
            # files to upload
            (tmp_dir_p / "a" / "b").mkdir(parents=True)
            (tmp_dir_p / "weights.txt").touch()
            (tmp_dir_p / "a" / "a.txt").touch()
            (tmp_dir_p / "a" / "b" / "b.txt").touch()
            (tmp_dir_p / "a" / "b" / ".bb").touch()
            expected_files = {
                "weights.txt",
                "a.txt",
                "b.txt",
                ".bb",
            }

            # files to ignore
            (tmp_dir_p / ".git").mkdir(parents=True)
            (tmp_dir_p / ".git" / "file").write_text("hidden git file")
            (tmp_dir_p / ".gitignore").write_text("none")

            (tmp_dir_p / "a" / ".git").mkdir(parents=True)
            (tmp_dir_p / "a" / "b" / ".git").mkdir(parents=True)
            (tmp_dir_p / "a" / "b" / ".git" / "abgit.txt").write_text("abgit")

            (tmp_dir_p / "a" / "b" / ".hidden").touch()

            (tmp_dir_p / "original" / "fp8").mkdir(parents=True)
            (tmp_dir_p / "original" / "fp8" / "weights").touch()
            (tmp_dir_p / "original" / "fp16").mkdir(parents=True)
            (tmp_dir_p / "original" / "fp16" / "weights").touch()

            # .git is already ignored by default
            ignore_patterns = [".gitignore", "*/.hidden", "original/"]
            model_upload(
                handle="metaresearch/testmodel/pytorch/withignore",
                local_model_dir=tmp_dir,
                ignore_patterns=ignore_patterns,
            )
            self.assertEqual(len(stub.shared_data.files), len(expected_files))
            self.assertEqual(set(stub.shared_data.files), expected_files)



import os
from typing import Optional

import requests

import kagglehub
from kagglehub.cache import MODELS_CACHE_SUBFOLDER, get_cached_archive_path
from kagglehub.handle import parse_model_handle
from tests.fixtures import BaseTestCase

from .server_stubs import model_download_stub as stub
from .server_stubs import serv
from .utils import create_test_cache

VERSIONED_MODEL_HANDLE = "metaresearch/llama-2/pyTorch/13b/3"
UNVERSIONED_MODEL_HANDLE = "metaresearch/llama-2/pyTorch/13b"
TEST_FILEPATH = "config.json"
TEST_CONTENTS = "{}"

EXPECTED_MODEL_SUBDIR = os.path.join(MODELS_CACHE_SUBFOLDER, "metaresearch", "llama-2", "pyTorch", "13b", "3")
EXPECTED_MODEL_SUBPATH = os.path.join(
    MODELS_CACHE_SUBFOLDER,
    "metaresearch",
    "llama-2",
    "pyTorch",
    "13b",
    "3",
    TEST_FILEPATH,
)


# Test cases for the ModelHttpResolver.
class TestHttpModelDownload(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app)

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def _download_model_and_assert_downloaded(
        self,
        d: str,
        model_handle: str,
        expected_subdir_or_subpath: str,
        expected_files: Optional[list[str]] = None,
        **kwargs,  # noqa: ANN003
    ) -> None:
        # Download the full model and ensure all files are there.
        model_path = kagglehub.model_download(model_handle, **kwargs)
        self.assertEqual(os.path.join(d, expected_subdir_or_subpath), model_path)
        if not expected_files:
            expected_files = ["config.json", "model.keras"]
        self.assertEqual(sorted(expected_files), sorted(os.listdir(model_path)))

        # Assert that the archive file has been deleted.
        archive_path = get_cached_archive_path(parse_model_handle(model_handle))
        self.assertFalse(os.path.exists(archive_path))

    def _download_test_file_and_assert_downloaded(self, d: str, model_handle: str, **kwargs) -> None:  # noqa: ANN003
        model_path = kagglehub.model_download(model_handle, path=TEST_FILEPATH, **kwargs)
        self.assertEqual(os.path.join(d, EXPECTED_MODEL_SUBPATH), model_path)
        with open(model_path) as model_file:
            self.assertEqual(TEST_CONTENTS, model_file.readline())

    def test_unversioned_model_download(self) -> None:
        with create_test_cache() as d:
            self._download_model_and_assert_downloaded(d, UNVERSIONED_MODEL_HANDLE, EXPECTED_MODEL_SUBDIR)

    def test_versioned_model_download(self) -> None:
        with create_test_cache() as d:
            self._download_model_and_assert_downloaded(d, VERSIONED_MODEL_HANDLE, EXPECTED_MODEL_SUBDIR)

    def test_model_archive_targz_download(self) -> None:
        with create_test_cache() as d:
            self._download_model_and_assert_downloaded(
                d,
                stub.TOO_MANY_FILES_FOR_PARALLEL_DOWNLOAD_HANDLE,
                f"{MODELS_CACHE_SUBFOLDER}/{stub.TOO_MANY_FILES_FOR_PARALLEL_DOWNLOAD_HANDLE}",
                expected_files=[f"{i}.txt" for i in range(1, 51)],
            )

    def test_model_archive_zip_download(self) -> None:
        with create_test_cache() as d:
            self._download_model_and_assert_downloaded(
                d,
                stub.ZIP_ARCHIVE_HANDLE,
                f"{MODELS_CACHE_SUBFOLDER}/{stub.ZIP_ARCHIVE_HANDLE}",
                expected_files=[f"model-{i}.txt" for i in range(1, 27)],
            )

    def test_versioned_model_full_download_with_file_already_cached(self) -> None:
        with create_test_cache() as d:
            # Download a single file first
            kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)
            self._download_model_and_assert_downloaded(d, VERSIONED_MODEL_HANDLE, EXPECTED_MODEL_SUBDIR)

    def test_unversioned_model_full_download_with_file_already_cached(self) -> None:
        with create_test_cache() as d:
            # Download a single file first
            kagglehub.model_download(UNVERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)
            self._download_model_and_assert_downloaded(d, UNVERSIONED_MODEL_HANDLE, EXPECTED_MODEL_SUBDIR)

    def test_unversioned_model_download_with_force_download(self) -> None:
        with create_test_cache() as d:
            self._download_model_and_assert_downloaded(
                d, UNVERSIONED_MODEL_HANDLE, EXPECTED_MODEL_SUBDIR, force_download=True
            )

    def test_versioned_model_download_with_force_download(self) -> None:
        with create_test_cache() as d:
            self._download_model_and_assert_downloaded(
                d, VERSIONED_MODEL_HANDLE, EXPECTED_MODEL_SUBDIR, force_download=True
            )

    def test_versioned_model_full_download_with_file_already_cached_and_force_download(self) -> None:
        with create_test_cache() as d:
            # Download a single file first
            kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)
            self._download_model_and_assert_downloaded(
                d, VERSIONED_MODEL_HANDLE, EXPECTED_MODEL_SUBDIR, force_download=True
            )

    def test_unversioned_model_full_download_with_file_already_cached_and_force_download(self) -> None:
        with create_test_cache() as d:
            # Download a single file first
            kagglehub.model_download(UNVERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)
            self._download_model_and_assert_downloaded(
                d, UNVERSIONED_MODEL_HANDLE, EXPECTED_MODEL_SUBDIR, force_download=True
            )

    def test_versioned_model_download_bad_archive(self) -> None:
        with create_test_cache():
            with self.assertRaises(ValueError):
                kagglehub.model_download(stub.INVALID_ARCHIVE_HANDLE)

    def test_versioned_model_download_with_path(self) -> None:
        with create_test_cache() as d:
            self._download_test_file_and_assert_downloaded(d, VERSIONED_MODEL_HANDLE)

    def test_unversioned_model_download_with_path(self) -> None:
        with create_test_cache() as d:
            self._download_test_file_and_assert_downloaded(d, UNVERSIONED_MODEL_HANDLE)

    def test_versioned_model_download_with_path_with_force_download(self) -> None:
        with create_test_cache() as d:
            self._download_test_file_and_assert_downloaded(d, VERSIONED_MODEL_HANDLE, force_download=True)

    def test_unversioned_model_download_with_path_with_force_download(self) -> None:
        with create_test_cache() as d:
            self._download_test_file_and_assert_downloaded(d, UNVERSIONED_MODEL_HANDLE, force_download=True)

    def test_versioned_model_download_already_cached(self) -> None:
        with create_test_cache() as d:
            # Download from server.
            kagglehub.model_download(VERSIONED_MODEL_HANDLE)

            # No internet, cache hit.
            model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE)

            self.assertEqual(os.path.join(d, EXPECTED_MODEL_SUBDIR), model_path)

    def test_versioned_model_download_with_path_already_cached(self) -> None:
        with create_test_cache() as d:
            kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)

            # No internet, cache hit.
            model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)

            self.assertEqual(os.path.join(d, EXPECTED_MODEL_SUBPATH), model_path)

    def test_versioned_model_download_already_cached_with_force_download_explicit_false(self) -> None:
        with create_test_cache() as d:
            kagglehub.model_download(VERSIONED_MODEL_HANDLE)

            # Not force downloaded, cache hit.
            model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE, force_download=False)

            self.assertEqual(os.path.join(d, EXPECTED_MODEL_SUBDIR), model_path)

    def test_versioned_model_download_with_path_already_cached_with_force_download_explicit_false(self) -> None:
        with create_test_cache() as d:
            kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)

            # Not force downloaded, cache hit.
            model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH, force_download=False)

            self.assertEqual(os.path.join(d, EXPECTED_MODEL_SUBPATH), model_path)


class TestHttpNoInternet(BaseTestCase):
    def test_versioned_model_download_already_cached_with_force_download(self) -> None:
        with create_test_cache():
            server = serv.start_server(stub.app)
            kagglehub.model_download(VERSIONED_MODEL_HANDLE)
            server.shutdown()

            # No internet should throw an error.
            with self.assertRaises(requests.exceptions.ConnectionError):
                kagglehub.model_download(VERSIONED_MODEL_HANDLE, force_download=True)

    def test_versioned_model_download_with_path_already_cached_with_force_download(self) -> None:
        with create_test_cache():
            server = serv.start_server(stub.app)
            kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)
            server.shutdown()

            # No internet should throw an error.
            with self.assertRaises(requests.exceptions.ConnectionError):
                kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH, force_download=True)



import re
from pathlib import Path

from flask import Flask, jsonify
from flask.typing import ResponseReturnValue
from flask_jwt_extended import JWTManager, create_access_token, jwt_required

app = Flask(__name__)
app.config["JWT_SECRET_KEY"] = "super-secret"

jwt = JWTManager(app)


@app.route("/setup/<path:text>", methods=["POST"])
def setup(text: str) -> ResponseReturnValue:
    """
    Workaround to setup token at a temp directory to simulate production.
    """
    token = create_access_token(identity="lastplacelarry")
    p = Path("/" + re.sub("^/", "", text))
    p.parent.mkdir(parents=True, exist_ok=True)
    with p.open("w") as f:
        f.write(token)
        f.write("\n")
        return jsonify({"token": token}, 200)


@app.route("/", methods=["HEAD"])
@jwt_required()
def head() -> ResponseReturnValue:
    return "", 200


@app.errorhandler(404)
def error(e: Exception):  # noqa: ANN201
    data = {"code": "404", "error": str(e)}
    return jsonify(data), 200


@app.route("/api/v1/hello", methods=["GET"])
@jwt_required()
def model_create() -> ResponseReturnValue:
    data = {"message": "Hello from test server!"}
    return jsonify(data), 200



import logging
from typing import Optional, Union

from kagglehub import registry
from kagglehub.gcs_upload import normalize_patterns, upload_files_and_directories
from kagglehub.handle import parse_model_handle
from kagglehub.logger import EXTRA_CONSOLE_BLOCK
from kagglehub.models_helpers import create_model_if_missing, create_model_instance_or_version

logger = logging.getLogger(__name__)

# Patterns that are always ignored for model uploading.
DEFAULT_IGNORE_PATTERNS = [".git/", "*/.git/", ".cache/", ".huggingface/"]


def model_download(handle: str, path: Optional[str] = None, *, force_download: Optional[bool] = False) -> str:
    """Download model files.

    Args:
        handle: (string) the model handle.
        path: (string) Optional path to a file within the model bundle.
        force_download: (bool) Optional flag to force download a model, even if it's cached.


    Returns:
        A string representing the path to the requested model files.
    """
    h = parse_model_handle(handle)
    logger.info(f"Downloading Model: {h.to_url()} ...", extra={**EXTRA_CONSOLE_BLOCK})
    return registry.model_resolver(h, path, force_download=force_download)


def model_upload(
    handle: str,
    local_model_dir: str,
    license_name: Optional[str] = None,
    version_notes: str = "",
    ignore_patterns: Optional[Union[list[str], str]] = None,
) -> None:
    """Upload model files.

    Args:
        handle: (str) the model handle.
        local_model_dir: (str) path to a file in a local directory.
        license_name: (str) model license.
        version_notes: (str, optional) model versions.
        ignore_patterns (str or list[str], optional):
            Additional ignore patterns to DEFAULT_IGNORE_PATTERNS.
            Files matching any of the patterns are not uploaded.
            Patterns are standard wildcards that can be matched by
            https://docs.python.org/3/library/fnmatch.html.
            Use a pattern ending with "/" to ignore the whole dir,
            e.g., ".git/" is equivalent to ".git/*".
    """
    # parse slug
    h = parse_model_handle(handle)
    logger.info(f"Uploading Model {h.to_url()} ...")
    if h.is_versioned():
        is_versioned_exception = "The model handle should not include the version"
        raise ValueError(is_versioned_exception)

    # Create the model if it doesn't already exist
    create_model_if_missing(h.owner, h.model)

    # Upload the model files to GCS

    tokens = upload_files_and_directories(
        local_model_dir,
        item_type="model",
        ignore_patterns=normalize_patterns(default=DEFAULT_IGNORE_PATTERNS, additional=ignore_patterns),
    )

    # Create a model instance if it doesn't exist, and create a new instance version if an instance exists
    create_model_instance_or_version(h, tokens, license_name, version_notes)



import unittest

from requests import HTTPError

from kagglehub import competition_download

from .utils import assert_files, create_test_cache

HANDLE = "titanic"


class TestCompetitionDownload(unittest.TestCase):
    def test_competition_succeeds(self) -> None:
        with create_test_cache():
            expected_files = [
                "gender_submission.csv",
                "test.csv",
                "train.csv",
            ]

            actual_path = competition_download(HANDLE)

            assert_files(self, actual_path, expected_files)

    def test_competition_competition_rules_accepted_succeeds(self) -> None:
        with create_test_cache():
            expected_files = [
                "sample_submission.csv",
                "test_identity.csv",
                "test_transaction.csv",
                "train_identity.csv",
                "train_transaction.csv",
            ]

            actual_path = competition_download("ieee-fraud-detection")

            assert_files(self, actual_path, expected_files)

    def test_competition_competition_rules_not_accepted_fails(self) -> None:
        # integrationtester bot has not accepted competiton rules
        with self.assertRaises(HTTPError) as e:
            competition_download("jane-street-market-prediction")
            self.assertEqual(e.exception.errno, 403)

    def test_competition_multiple_files(self) -> None:
        with create_test_cache():
            file_paths = [
                "gender_submission.csv",
                "test.csv",
                "train.csv",
            ]
            for p in file_paths:
                actual_path = competition_download(HANDLE, path=p)
                assert_files(self, actual_path, [p])

    def test_competition_with_incorrect_file_path(self) -> None:
        incorrect_path = "nonxisten/Test"
        with self.assertRaises(HTTPError) as e:
            competition_download(HANDLE, path=incorrect_path)
            self.assertEqual(e.exception.errno, 403)



from kagglehub.handle import parse_competition_handle, parse_dataset_handle, parse_model_handle
from tests.fixtures import BaseTestCase


class TestHandle(BaseTestCase):
    def test_versioned_model_handle(self) -> None:
        handle = "google/bert/tensorFlow2/answer-equivalence-bem/3"
        h = parse_model_handle(handle)

        self.assertEqual("google", h.owner)
        self.assertEqual("bert", h.model)
        self.assertEqual("tensorFlow2", h.framework)
        self.assertEqual("answer-equivalence-bem", h.variation)
        self.assertEqual(3, h.version)
        self.assertTrue(h.is_versioned())
        self.assertEqual(handle, str(h))

    def test_unversioned_model_handle(self) -> None:
        handle = "google/bert/tensorFlow2/answer-equivalence-bem"
        h = parse_model_handle(handle)
        self.assertEqual("google", h.owner)
        self.assertEqual("bert", h.model)
        self.assertEqual("tensorFlow2", h.framework)
        self.assertEqual("answer-equivalence-bem", h.variation)
        self.assertEqual(None, h.version)
        self.assertFalse(h.is_versioned())
        self.assertEqual(handle, str(h))

    def test_invalid_model_handle(self) -> None:
        with self.assertRaises(ValueError):
            parse_model_handle("invalid")

    def test_invalid_version_model_handle(self) -> None:
        with self.assertRaises(ValueError):
            parse_model_handle("google/bert/tensorFlow2/answer-equivalence-bem/invalid-version-number")

    def test_unversioned_dataset_handle(self) -> None:
        handle = "owner/dataset"
        h = parse_dataset_handle(handle)

        self.assertEqual("owner", h.owner)
        self.assertEqual("dataset", h.dataset)
        self.assertEqual(None, h.version)
        self.assertFalse(h.is_versioned())
        self.assertEqual(handle, str(h))

    def test_invalid_dataset_handle(self) -> None:
        with self.assertRaises(ValueError):
            parse_dataset_handle("a-single-part")

    def test_versioned_dataset_handle(self) -> None:
        handle = "owner/versionedDataset/versions/2"
        h = parse_dataset_handle(handle)

        self.assertEqual("owner", h.owner)
        self.assertEqual("versionedDataset", h.dataset)
        self.assertEqual(2, h.version)
        self.assertTrue(h.is_versioned())
        self.assertEqual(handle, str(h))

    def test_invalid_competition_handle(self) -> None:
        with self.assertRaises(ValueError):
            parse_competition_handle("ownerSlugNotNeeded/onlyCompetitionSlug")

    def test_competition_handle(self) -> None:
        handle = "titanic"

        h = parse_competition_handle(handle)

        self.assertEqual("titanic", h.competition)



import unittest

from kagglehub import whoami
from kagglehub.exceptions import UnauthenticatedError

from .utils import unauthenticated


class TestUtils(unittest.TestCase):
    def test_unauthenticated(self) -> None:
        with unauthenticated():
            with self.assertRaises(UnauthenticatedError):
                whoami()



import os
from pathlib import Path
from tempfile import TemporaryDirectory

from kagglehub.datasets import dataset_upload
from kagglehub.gcs_upload import MAX_FILES_TO_UPLOAD, TEMP_ARCHIVE_FILE
from tests.fixtures import BaseTestCase

from .server_stubs import dataset_upload_stub as stub
from .server_stubs import serv

TEMP_TEST_FILE = "temp-test-file"


class TestDatasetUpload(BaseTestCase):
    def setUp(self) -> None:
        stub.reset()

    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app)

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def test_dataset_upload_with_invalid_handle(self) -> None:
        with self.assertRaises(ValueError):
            with TemporaryDirectory() as temp_dir:
                test_filepath = Path(temp_dir) / TEMP_TEST_FILE
                test_filepath.touch()  # Create a temporary file in the temporary directory
                dataset_upload("invalid/invalid/invalid", temp_dir)

    def test_dataset_upload_with_valid_handle(self) -> None:
        with TemporaryDirectory() as temp_dir:
            test_filepath = Path(temp_dir) / TEMP_TEST_FILE
            test_filepath.touch()  # Create a temporary file in the temporary directory
            dataset_upload("jeward/newDataset", temp_dir, "dataset-type")
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_TEST_FILE, stub.shared_data.files)

    def test_dataset_upload_instance_with_nested_directories(self) -> None:
        with TemporaryDirectory() as temp_dir:
            # Create a nested directory structure
            nested_dir = Path(temp_dir) / "nested"
            nested_dir.mkdir()
            # Create a temporary file in the nested directory
            test_filepath = nested_dir / TEMP_TEST_FILE
            test_filepath.touch()
            dataset_upload("jeward/newDataset", temp_dir, "dataset_type")
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_TEST_FILE, stub.shared_data.files)

    def test_dataset_upload_with_too_many_files(self) -> None:
        with TemporaryDirectory() as temp_dir:
            # Create more than 50 temporary files in the directory
            for i in range(MAX_FILES_TO_UPLOAD + 1):
                test_filepath = Path(temp_dir) / f"temp_test_file_{i}"
                test_filepath.touch()
            dataset_upload("jeward/newDataset", temp_dir, "dataset_type")
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_ARCHIVE_FILE, stub.shared_data.files)

    def test_dataset_upload_resumable(self) -> None:
        stub.simulate_308(state=True)  # Enable simulation of 308 response for this test
        with TemporaryDirectory() as temp_dir:
            test_filepath = Path(temp_dir) / TEMP_TEST_FILE
            test_filepath.touch()
            with open(test_filepath, "wb") as f:
                f.write(os.urandom(1000))
            dataset_upload("jeward/newDataset", temp_dir, "dataset_type")
            self.assertGreaterEqual(stub.shared_data.blob_request_count, 1)
            self.assertEqual(len(stub.shared_data.files), 1)
            self.assertIn(TEMP_TEST_FILE, stub.shared_data.files)



import os
from collections.abc import Generator
from contextlib import contextmanager
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Any
from unittest import mock

from flask import Flask, jsonify, request
from flask.typing import ResponseReturnValue

from kagglehub.clients import (
    KAGGLE_DATA_PROXY_TOKEN_ENV_VAR_NAME,
    KAGGLE_JWT_TOKEN_ENV_VAR_NAME,
)
from kagglehub.env import KAGGLE_NOTEBOOK_ENV_VAR_NAME
from kagglehub.kaggle_cache_resolver import KAGGLE_CACHE_MOUNT_FOLDER_ENV_VAR_NAME

app = Flask(__name__)

LATEST_MODEL_VERSION = 2
LATEST_DATASET_VERSION = 2


@app.route("/", methods=["HEAD"])
def head() -> ResponseReturnValue:
    return "", 200


@app.errorhandler(404)
def error(e: Exception):  # noqa: ANN201
    data = {"code": "404", "error": str(e)}
    return jsonify(data), 200


@app.route("/kaggle-jwt-handler/AttachDatasourceUsingJwtRequest", methods=["POST"])
def attach_datasource_using_jwt_request() -> ResponseReturnValue:
    data = request.get_json()
    if "modelRef" in data:
        model_ref = data["modelRef"]
        version_number = LATEST_MODEL_VERSION
        if "VersionNumber" in model_ref:
            version_number = model_ref["VersionNumber"]
        mount_slug = f"{model_ref['ModelSlug']}/{model_ref['Framework']}/{model_ref['InstanceSlug']}/{version_number}"
        # # Load the files
        cache_mount_folder = os.getenv(KAGGLE_CACHE_MOUNT_FOLDER_ENV_VAR_NAME)
        base_path = f"{cache_mount_folder}/{mount_slug}"
        os.makedirs(base_path, exist_ok=True)
        Path(f"{base_path}/config.json").touch()
        if version_number == LATEST_MODEL_VERSION:
            # The latest version has an extra file.
            Path(f"{base_path}/model.keras").touch()
        data = {
            "wasSuccessful": True,
            "result": {
                "mountSlug": mount_slug,
            },
        }
        return jsonify(data), 200
    elif "datasetRef" in data:
        dataset_ref = data["datasetRef"]
        version_number = LATEST_DATASET_VERSION
        if "VersionNumber" in dataset_ref:
            version_number = dataset_ref["VersionNumber"]
        mount_slug = f"{dataset_ref['DatasetSlug']}"
        # # Load the files
        cache_mount_folder = os.getenv(KAGGLE_CACHE_MOUNT_FOLDER_ENV_VAR_NAME)
        base_path = f"{cache_mount_folder}/{mount_slug}"
        os.makedirs(base_path, exist_ok=True)
        Path(f"{base_path}/foo.txt").touch()
        if version_number == LATEST_DATASET_VERSION:
            # The latest version has an extra file.
            Path(f"{base_path}/bar.csv").touch()
        data = {
            "wasSuccessful": True,
            "result": {
                "mountSlug": mount_slug,
            },
        }
        return jsonify(data), 200
    elif "competitionRef" in data:
        competition_ref = data["competitionRef"]
        mount_slug = f"{competition_ref['CompetitionSlug']}"
        # # Load the files
        cache_mount_folder = os.getenv(KAGGLE_CACHE_MOUNT_FOLDER_ENV_VAR_NAME)
        base_path = f"{cache_mount_folder}/{mount_slug}"
        os.makedirs(base_path, exist_ok=True)
        Path(f"{base_path}/foo.txt").touch()
        Path(f"{base_path}/bar.csv").touch()
        data = {
            "wasSuccessful": True,
            "result": {
                "mountSlug": mount_slug,
            },
        }
        return jsonify(data), 200
    else:
        return jsonify(data), 500


@contextmanager
def create_env() -> Generator[Any, Any, Any]:
    with TemporaryDirectory() as cache_mount_folder:
        with mock.patch.dict(
            os.environ,
            {
                KAGGLE_NOTEBOOK_ENV_VAR_NAME: "Interactive",
                KAGGLE_JWT_TOKEN_ENV_VAR_NAME: "foo jwt token",
                KAGGLE_DATA_PROXY_TOKEN_ENV_VAR_NAME: "foo proxy token",
                KAGGLE_CACHE_MOUNT_FOLDER_ENV_VAR_NAME: cache_mount_folder,
            },
        ):
            yield



import unittest

from requests import HTTPError

from kagglehub import dataset_download

from .utils import assert_files, create_test_cache, unauthenticated

UNVERSIONED_HANDLE = "ryanholbrook/dl-course-data"
HANDLE = "ryanholbrook/dl-course-data/versions/5"


class TestDatasetDownload(unittest.TestCase):
    def test_dataset_versioned_succeeds(self) -> None:
        with create_test_cache():
            actual_path = dataset_download(HANDLE)

            expected_files = [
                "abalone.csv",
                "candy.csv",
                "cereal.csv",
                "concrete.csv",
                "diamonds.csv",
                "forestfires.csv",
                "fuel.csv",
                "hotel.csv",
                "housing.csv",
                "ion.csv",
                "red-wine.csv",
                "songs.csv",
                "spotify.csv",
            ]
            assert_files(self, actual_path, expected_files)

    def test_dataset_unversioned_succeeds(self) -> None:
        with create_test_cache():
            actual_path = dataset_download(UNVERSIONED_HANDLE)

            expected_files = [
                "abalone.csv",
                "candy.csv",
                "cereal.csv",
                "concrete.csv",
                "diamonds.csv",
                "forestfires.csv",
                "fuel.csv",
                "hotel.csv",
                "housing.csv",
                "ion.csv",
                "red-wine.csv",
                "songs.csv",
                "spotify.csv",
            ]
            assert_files(self, actual_path, expected_files)

    def test_download_private_dataset_succeeds(self) -> None:
        with create_test_cache():
            actual_path = dataset_download("integrationtester/kagglehub-test-private-dataset")

            expected_files = [
                "private.txt",
            ]

            assert_files(self, actual_path, expected_files)

    def test_download_multiple_files(self) -> None:
        with create_test_cache():
            file_paths = ["abalone.csv", "diamonds.csv", "red-wine.csv"]
            for p in file_paths:
                actual_path = dataset_download(HANDLE, path=p)
                assert_files(self, actual_path, [p])

    def test_download_with_incorrect_file_path(self) -> None:
        incorrect_path = "nonexistent/file/path"
        with self.assertRaises(HTTPError):
            dataset_download(HANDLE, path=incorrect_path)

    def test_public_dataset_with_unauthenticated_succeeds(self) -> None:
        with create_test_cache():
            with unauthenticated():
                actual_path = dataset_download(UNVERSIONED_HANDLE)

                expected_files = [
                    "abalone.csv",
                    "candy.csv",
                    "cereal.csv",
                    "concrete.csv",
                    "diamonds.csv",
                    "forestfires.csv",
                    "fuel.csv",
                    "hotel.csv",
                    "housing.csv",
                    "ion.csv",
                    "red-wine.csv",
                    "songs.csv",
                    "spotify.csv",
                ]
                assert_files(self, actual_path, expected_files)
